# Install Embedded ASM
Embedded ASM is a minimal variant of ASM that is installed as part of Apigee hybrid installation for setting up ingress infrastructure.

Design: [go/embedded-asm-apigee-design](http://go/embedded-asm-apigee-design)

Slides: [go/embedded-asm-apigee-slides](http://go/embedded-asm-apigee-slides)

Overview: [go/embedded-asm-apigee-overview](http://go/embedded-asm-apigee-overview)

## Pre-req
Follow instructions here: [go/hybrid-dev-create](http://go/hybrid-dev-create) to set up a hybrid Apigee Org for experimenting purpose.

## Install / Development
Build `apigeectl` with local changes (Optional if you already have latest apigeectl)
```
GO111MODULE=off
go build -o apigeectl
```
Follow the instructions [here](https://g3doc.corp.google.com/company/teams/edge/engDocs/engineering/sdlc/hybrid-automation.md#deploy-hybrid-2) to deploy hybrid on the cluster. From the job logs capture([screenshot](https://screenshot.googleplex.com/YXuj7iC3pgegAZg) on how to find the yaml) the overrides yaml file and store it in `overrides-easm.yaml`.
The file needs below replacements
1. `sslKeyPath` with https://valentine.corp.google.com/#/show/1569542172151890?tab=metadata
2. `sslCertPath` with https://valentine.corp.google.com/#/show/1569542127800021?tab=metadata
3. Append below section to the file
```
istiod:
  image:
    url: gcr.io/gke-release/asm/pilot
    tag: 1.11.2-asm.17
```
4. (Optional) If you have a custom operator image then update the below section. This [README](https://edge-internal.git.corp.google.com/k8s-controllers/#build-and-push-image) talks about how to generate custom controller image.
```
ao:
  image:
    url: us.gcr.io/apigee-eda-experiments/apigee-controller
    tag: gauthamvk
```
Generate the service account cred files locally using below cmds ([Ref](https://cloud.google.com/apigee/docs/hybrid/v1.6/create-service-account))
```
gcloud config set project PROJECT_ID
./tools/create-service-account --env prod
```
Replace the respective `.json` file paths with generated files from previous cmd. Use mart cred files for apigee connect.

Use `--skip-asm-validation=true` so that you don't have to install ASM in the cluster.
Use `--install-istiod=true` to install istiod.
```
./apigeectl init -f overrides-easm.yaml --skip-asm-validation=true --install-istiod=true
```
In addition to other stuff, it creates a new `apigee-ingressgateway-manager` deployment in `test-1` namespace.

Sample `overrides-easm.yaml` below
```
ao:
  nodeSelector:
    key: cloud.google.com/gke-nodepool
    value: apigee-runtime
redis:
cassandra:
  heapNewSize: 100M
  maxHeapSize: 512M
  nodeSelector:
    key: cloud.google.com/gke-nodepool
    value: apigee-data
  replicaCount: 3
  resources:
    requests:
      cpu: 500m
      memory: 1Gi
  rack: rc-1
  datacenter: us-west1
  storage:
    capacity: 10Gi
    storageclass: apigee-sc
connectAgent:
  replicaCountMax: 5
  replicaCountMin: 3
  server: apigeeconnect.googleapis.com:443
  serviceAccountPath: service-accounts/gauthamvk-hybrid-dev-apigee-mart.json
watcher:
  serviceAccountPath: service-accounts/gauthamvk-hybrid-dev-apigee-watcher.json
contractProvider: https://apigee.googleapis.com
gcp:
  projectID: gauthamvk-hybrid-dev
  region: us-west1
gcpProjectID: gauthamvk-hybrid-dev
gcpRegion: us-west1
imagePullSecrets:
- name: gcr-cred
k8sCluster:
  name: hybrid-test
  region: us-west1
k8sClusterName: hybrid-test
instanceID: hybrid-test-fpll
kmsEncryptionKey: azhxxx
kvmEncryptionKey: azhxxx
logger:
  enable: false
  serviceAccountPath: service-accounts/gauthamvk-hybrid-dev-apigee-logger.json
mart:
  nodeSelector:
    key: cloud.google.com/gke-nodepool
    value: apigee-runtime
  replicaCountMax: 2
  replicaCountMin: 2
  serviceAccountPath: service-accounts/gauthamvk-hybrid-dev-apigee-mart.json
  #sslCertPath: /app/source/certs/mart.crt
  #sslKeyPath: /app/source/certs/mart.key
metrics:
  serviceAccountPath: service-accounts/gauthamvk-hybrid-dev-apigee-metrics.json
namespace: test-1
org: gauthamvk-hybrid-dev
runtime:
  nodeSelector:
    key: cloud.google.com/gke-nodepool
    value: apigee-runtime
  replicaCountMax: 12
  replicaCountMin: 2
synchronizer:
  nodeSelector:
    key: cloud.google.com/gke-nodepool
    value: apigee-runtime
  replicaCountMax: 4
  replicaCountMin: 1
udca:
  nodeSelector:
    key: cloud.google.com/gke-nodepool
    value: apigee-runtime
  replicaCountMax: 12
  replicaCountMin: 2
validateServiceAccounts: true
istio:
  nodeSelector:
    key: cloud.google.com/gke-nodepool
    value: apigee-runtime
virtualhosts:
- name: env-group
  sslCertPath: ../valentine/runtime.crt
  sslKeyPath: ../valentine/runtime.key
envs:
- name: test
  serviceAccountPaths:
    synchronizer: service-accounts/gauthamvk-hybrid-dev-apigee-synchronizer.json
    udca: service-accounts/gauthamvk-hybrid-dev-apigee-udca.json
    runtime: service-accounts/gauthamvk-hybrid-dev-apigee-runtime.json
  cacheEncryptionKey: azhzxxx
  kvmEncryptionKey: azhzxxx
istiod:
  image:
    url: gcr.io/gke-release/asm/pilot
    tag: 1.11.2-asm.17
```
## Verify
```
$ kubectl get pods -n test-1
NAME                                                              READY   STATUS      RESTARTS   AGE
...
apigee-ingressgateway-manager-55b4bf4685-2sqks                                    3/3     Running     1          16h
apigee-ingressgateway-manager-55b4bf4685-jm9p6                                    3/3     Running     1          16h
...
```
You must find apigee-ingressgateway-manager PODs in running state, if not check the POD logs for debugging.

## Sample
Deploy a sample ingress and helloworld application.
```
kubectl apply -f ../../examples/embeddedasm/helloworld
```
Verify
```
kubectl exec -it -n test-1 <ingress-pod-name> bash -- curl 0:8080/hello -k -v -H "Host: httpbin.local"
```
```
kubectl exec -it -n test-1 <ingress-pod-name> bash -- curl https://httpbin.local:8443/hello -k -v --resolve "httpbin.local:8443:127.0.0.1"
```
